import { __assign } from "tslib";
import React, { useState, useEffect } from 'react';
import { rgb2cmyk, cmykToRgb } from './converters';
import { formatInputValues } from './formatters';
import { usePicker } from './context';
import { inputWrap, inputLabel } from './style';
var tc = require("tinycolor2");
var Inputs = function () {
    var _a = useState(''), disable = _a[0], setDisable = _a[1];
    var _b = usePicker(), handleChange = _b.handleChange, tinyColor = _b.tinyColor, r = _b.r, g = _b.g, b = _b.b, opacity = _b.opacity, inputType = _b.inputType;
    var hex = tinyColor.toHex();
    var _c = useState(hex), newHex = _c[0], setNewHex = _c[1];
    useEffect(function () {
        if (disable !== 'hex') {
            setNewHex(hex);
        }
    }, [tinyColor, disable, hex]);
    var handleHex = function (e) {
        var tinyHex = tc(e.target.value);
        setNewHex(e.target.value);
        if (tinyHex.isValid()) {
            var _a = tinyHex.toRgb(), r_1 = _a.r, g_1 = _a.g, b_1 = _a.b;
            var newColor = "rgba(".concat(r_1, ", ").concat(g_1, ", ").concat(b_1, ", ").concat(opacity, ")");
            handleChange(newColor);
        }
    };
    return (React.createElement("div", { style: { display: 'flex', justifyContent: 'space-between', paddingTop: 14 } },
        React.createElement("div", { style: { width: '23%' } },
            React.createElement("input", { style: __assign({}, inputWrap), value: newHex, onChange: function (e) { return handleHex(e); }, onFocus: function () { return setDisable('hex'); }, onBlur: function () { return setDisable(''); } }),
            React.createElement("div", { style: __assign({}, inputLabel) }, "HEX")),
        inputType === 'hsl' && React.createElement(HSLInputs, null),
        inputType === 'rgb' && React.createElement(RGBInputs, null),
        inputType === 'hsv' && React.createElement(HSVInputs, null),
        inputType === 'cmyk' && React.createElement(CMKYInputs, null),
        React.createElement(Input, { value: opacity * 100, callback: function (newVal) { return handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(newVal / 100, ")")); }, label: 'A' })));
};
export default Inputs;
var RGBInputs = function () {
    var _a = usePicker(), handleChange = _a.handleChange, r = _a.r, g = _a.g, b = _a.b, opacity = _a.opacity;
    var handleRgb = function (_a) {
        var r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Input, { value: r, callback: function (newVal) { return handleRgb({ r: newVal, g: g, b: b }); }, label: 'R', max: 255 }),
        React.createElement(Input, { value: g, callback: function (newVal) { return handleRgb({ r: r, g: newVal, b: b }); }, label: 'G', max: 255 }),
        React.createElement(Input, { value: b, callback: function (newVal) { return handleRgb({ r: r, g: g, b: newVal }); }, label: 'B', max: 255 })));
};
var HSLInputs = function () {
    var _a = usePicker(), handleChange = _a.handleChange, s = _a.s, l = _a.l, internalHue = _a.internalHue, opacity = _a.opacity, setInternalHue = _a.setInternalHue;
    var handleH = function (h, s, l) {
        setInternalHue(h);
        var _a = tc({ h: h, s: s, l: l }).toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    var handleSl = function (value) {
        var _a = tc(value).toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Input, { value: round(internalHue), callback: function (newVal) { return handleH(newVal, s, l); }, label: 'H', max: 360 }),
        React.createElement(Input, { value: round(s * 100), callback: function (newVal) { return handleSl({ h: internalHue, s: newVal, l: l }); }, label: 'S' }),
        React.createElement(Input, { value: round(l * 100), callback: function (newVal) { return handleSl({ h: internalHue, s: s, l: newVal }); }, label: 'L' })));
};
var HSVInputs = function () {
    var _a = usePicker(), handleChange = _a.handleChange, hsvS = _a.hsvS, hsvV = _a.hsvV, internalHue = _a.internalHue, setInternalHue = _a.setInternalHue, opacity = _a.opacity;
    var handleH = function (h, s, v) {
        setInternalHue(h);
        var _a = tc({ h: h, s: s, v: v }).toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    var handleSV = function (value) {
        var _a = tc(value).toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Input, { value: round(internalHue), callback: function (newVal) { return handleH(newVal, hsvS, hsvV); }, label: 'H', max: 360 }),
        React.createElement(Input, { value: round(hsvS * 100), callback: function (newVal) { return handleSV({ h: internalHue, s: newVal, v: hsvV }); }, label: 'S' }),
        React.createElement(Input, { value: round(hsvV * 100), callback: function (newVal) { return handleSV({ h: internalHue, s: hsvS, v: newVal }); }, label: 'V' })));
};
var CMKYInputs = function () {
    var _a = usePicker(), handleChange = _a.handleChange, r = _a.r, g = _a.g, b = _a.b, opacity = _a.opacity;
    var _b = rgb2cmyk(r, g, b), c = _b.c, m = _b.m, k = _b.k, y = _b.y;
    var handleCmyk = function (value) {
        var _a = cmykToRgb(value), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")"));
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Input, { value: round(c * 100), callback: function (newVal) { return handleCmyk({ c: newVal, m: m, y: y, k: k }); }, label: 'C' }),
        React.createElement(Input, { value: round(m * 100), callback: function (newVal) { return handleCmyk({ c: c, m: newVal, y: y, k: k }); }, label: 'M' }),
        React.createElement(Input, { value: round(y * 100), callback: function (newVal) { return handleCmyk({ c: c, m: m, y: newVal, k: k }); }, label: 'Y' }),
        React.createElement(Input, { value: round(k * 100), callback: function (newVal) { return handleCmyk({ c: c, m: m, y: y, k: newVal }); }, label: 'K' })));
};
var Input = function (_a) {
    var value = _a.value, callback = _a.callback, _b = _a.max, max = _b === void 0 ? 100 : _b, label = _a.label;
    var _c = useState(value), temp = _c[0], setTemp = _c[1];
    var inputType = usePicker().inputType;
    useEffect(function () {
        setTemp(value);
    }, [value]);
    var onChange = function (e) {
        var newVal = formatInputValues(parseFloat(e.target.value), 0, max);
        setTemp(newVal);
        callback(newVal);
    };
    return (React.createElement("div", { style: { width: inputType === 'cmyk' ? '14.9%' : '18%' } },
        React.createElement("input", { style: __assign({}, inputWrap), value: temp, onChange: function (e) { return onChange(e); } }),
        React.createElement("div", { style: __assign({}, inputLabel) }, label)));
};
var round = function (val) {
    return Math.round(val, 2);
};
//# sourceMappingURL=Inputs.js.map