import { __assign, __spreadArray } from "tslib";
import React, { createContext, useContext, useState, useEffect } from 'react';
import { computePickerPosition, getGradientType, computeSquareXY, getDegrees, getNewHsl, getHandleValue, isUpperCase } from './utils';
import { low, high, getColors } from './formatters';
import { config } from './constants';
import PropTypes from 'prop-types';
var tinycolor = require("tinycolor2");
var crossSize = config.crossSize;
var PickerContext = createContext();
export default function PickerContextWrapper(_a) {
    var children = _a.children, bounds = _a.bounds, value = _a.value, onChange = _a.onChange, squareSize = _a.squareSize, squareHeight = _a.squareHeight;
    var offsetLeft = bounds === null || bounds === void 0 ? void 0 : bounds.x;
    var isGradient = value === null || value === void 0 ? void 0 : value.includes('gradient');
    var gradientType = getGradientType(value);
    var degrees = getDegrees(value);
    var degreeStr = gradientType === 'linear-gradient' ? "".concat(degrees, "deg") : 'circle';
    var colors = getColors(value);
    var indexedColors = colors === null || colors === void 0 ? void 0 : colors.map(function (c, i) { return (__assign(__assign({}, c), { index: i })); });
    var currentColorObj = (indexedColors === null || indexedColors === void 0 ? void 0 : indexedColors.filter(function (c) { return isUpperCase(c.value); })[0]) || indexedColors[0];
    var currentColor = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.value;
    var selectedColor = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.index;
    var currentLeft = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.left;
    var _b = useState(tinycolor(currentColor)), tinyColor = _b[0], setTinyColor = _b[1];
    var _c = useState('rgb'), inputType = _c[0], setInputType = _c[1];
    var _d = tinyColor.toRgb(), r = _d.r, g = _d.g, b = _d.b, opacity = _d.a;
    var _e = tinyColor.toHsl(), h = _e.h, s = _e.s, l = _e.l;
    var _f = tinyColor.toHsv(), hsvS = _f.s, hsvV = _f.v;
    var _g = useState(Math.round(h)), internalHue = _g[0], setInternalHue = _g[1];
    var hue = Math.round(h);
    var _h = computeSquareXY([hue, s, l], squareSize, squareHeight), x = _h[0], y = _h[1];
    var _j = useState([]), previousColors = _j[0], setPreviousColors = _j[1];
    var _k = useState([]), previousGraidents = _k[0], setPreviousGradients = _k[1];
    useEffect(function () {
        setTinyColor(tinycolor(currentColor));
        setInternalHue(hue);
    }, [currentColor, hue]);
    useEffect(function () {
        if (isGradient) {
            setPreviousGradients(__spreadArray([value], previousGraidents === null || previousGraidents === void 0 ? void 0 : previousGraidents.slice(0, 4), true));
        }
        else {
            if (tinycolor(value).isValid()) {
                setPreviousColors(__spreadArray([value], previousColors === null || previousColors === void 0 ? void 0 : previousColors.slice(0, 4), true));
            }
        }
        //eslint-disable-next-line
    }, [value]);
    var createGradientStr = function (newColors) {
        var sorted = newColors.sort(function (a, b) { return a.left - b.left; });
        var colorString = sorted === null || sorted === void 0 ? void 0 : sorted.map(function (cc) { return "".concat(cc === null || cc === void 0 ? void 0 : cc.value, " ").concat(cc.left, "%"); });
        onChange("".concat(gradientType, "(").concat(degreeStr, ", ").concat(colorString.join(', '), ")"));
    };
    var handleGradient = function (newColor, left) {
        if (left === void 0) { left = currentLeft; }
        var remaining = colors === null || colors === void 0 ? void 0 : colors.filter(function (c) { return !isUpperCase(c.value); });
        var newColors = __spreadArray([{ value: newColor.toUpperCase(), left: left }], remaining, true);
        createGradientStr(newColors);
    };
    var handleChange = function (newColor) {
        if (isGradient) {
            handleGradient(newColor);
        }
        else {
            onChange(newColor);
        }
    };
    var handleOpacity = function (e) {
        var newO = getHandleValue(e) / 100;
        var newColor = "rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(newO, ")");
        handleChange(newColor);
    };
    var handleHue = function (e) {
        var newHue = getHandleValue(e) * 3.6;
        var newHsl = getNewHsl(newHue, s, l, opacity, setInternalHue);
        handleChange(newHsl);
    };
    var handleColor = function (e, ctx) {
        var _a = computePickerPosition(e, squareHeight), x = _a[0], y = _a[1];
        var x1 = Math.min(x + crossSize / 2, squareSize - 1);
        var y1 = Math.min(y + crossSize / 2, squareHeight - 1);
        var _b = ctx.getImageData(x1, y1, 1, 1).data, r = _b[0], g = _b[1], b = _b[2];
        var newColor = "rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(opacity, ")");
        handleChange(newColor);
    };
    var setSelectedColor = function (index) {
        var newGradStr = colors === null || colors === void 0 ? void 0 : colors.map(function (cc, i) { return (__assign(__assign({}, cc), { value: i === index ? high(cc) : low(cc) })); });
        createGradientStr(newGradStr);
    };
    var addPoint = function (e) {
        var left = getHandleValue(e, offsetLeft);
        var newColors = __spreadArray(__spreadArray([], colors.map(function (c) { return (__assign(__assign({}, c), { value: low(c) })); }), true), [{ value: currentColor, left: left }], false);
        createGradientStr(newColors);
    };
    var deletePoint = function () {
        if ((colors === null || colors === void 0 ? void 0 : colors.length) > 2) {
            var remaining = colors === null || colors === void 0 ? void 0 : colors.filter(function (rc, i) { return i !== selectedColor; });
            createGradientStr(remaining);
        }
    };
    var pickerState = {
        x: x,
        y: y,
        s: s,
        l: l,
        r: r,
        g: g,
        b: b,
        hue: hue,
        hsvS: hsvS,
        hsvV: hsvV,
        value: value,
        colors: colors,
        degrees: degrees,
        opacity: opacity,
        onChange: onChange,
        addPoint: addPoint,
        inputType: inputType,
        tinyColor: tinyColor,
        handleHue: handleHue,
        isGradient: isGradient,
        offsetLeft: offsetLeft,
        squareSize: squareSize,
        handleColor: handleColor,
        currentLeft: currentLeft,
        deletePoint: deletePoint,
        internalHue: internalHue,
        squareHeight: squareHeight,
        setInputType: setInputType,
        gradientType: gradientType,
        handleChange: handleChange,
        currentColor: currentColor,
        selectedColor: selectedColor,
        handleOpacity: handleOpacity,
        setInternalHue: setInternalHue,
        previousColors: previousColors,
        handleGradient: handleGradient,
        setSelectedColor: setSelectedColor,
        previousGraidents: previousGraidents,
    };
    return React.createElement(PickerContext.Provider, { value: pickerState }, children);
}
export function usePicker() {
    return useContext(PickerContext);
}
PickerContextWrapper.propTypes = {
    children: PropTypes.node,
    bounds: PropTypes.object,
    value: PropTypes.string,
    onChange: PropTypes.func,
};
//# sourceMappingURL=context.js.map