import { __assign, __spreadArray } from "tslib";
import { useState, useEffect } from 'react';
import { getGradientType, getDegrees, isUpperCase } from './utils';
import { low, high, getColors, formatInputValues } from './formatters';
import { rgb2cmyk } from './converters';
import { config } from './constants';
var defaultColor = config.defaultColor, defaultGradient = config.defaultGradient;
var tc = require("tinycolor2");
export var useColorPicker = function (value, onChange) {
    var isGradient = value === null || value === void 0 ? void 0 : value.includes('gradient');
    var gradientType = getGradientType(value);
    var degrees = getDegrees(value);
    var degreeStr = gradientType === 'linear-gradient' ? "".concat(degrees, "deg") : 'circle';
    var colors = getColors(value);
    var indexedColors = colors === null || colors === void 0 ? void 0 : colors.map(function (c, i) { return (__assign(__assign({}, c), { index: i })); });
    var currentColorObj = (indexedColors === null || indexedColors === void 0 ? void 0 : indexedColors.filter(function (c) { return isUpperCase(c.value); })[0]) || indexedColors[0];
    var currentColor = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.value;
    var selectedPoint = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.index;
    var currentLeft = currentColorObj === null || currentColorObj === void 0 ? void 0 : currentColorObj.left;
    var _a = useState([]), previousColors = _a[0], setPreviousColors = _a[1];
    var tiny = tc(currentColor);
    var _b = tiny.toRgb(), r = _b.r, g = _b.g, b = _b.b, a = _b.a;
    var _c = tiny.toHsl(), h = _c.h, s = _c.s, l = _c.l;
    useEffect(function () {
        var _a;
        if ((_a = tc(currentColor)) === null || _a === void 0 ? void 0 : _a.isValid()) {
            setPreviousColors(__spreadArray([currentColor], previousColors === null || previousColors === void 0 ? void 0 : previousColors.slice(0, 19), true));
        }
    }, [currentColor]);
    var setLinear = function () {
        var remaining = value.split(/,(.+)/)[1];
        onChange("linear-gradient(90deg, ".concat(remaining));
    };
    var setRadial = function () {
        var remaining = value.split(/,(.+)/)[1];
        onChange("radial-gradient(circle, ".concat(remaining));
    };
    var setDegrees = function (newDegrees) {
        var remaining = value.split(/,(.+)/)[1];
        onChange("linear-gradient(".concat(formatInputValues(newDegrees, 0, 360), "deg, ").concat(remaining));
        if (gradientType !== 'linear-gradient') {
            console.log('Warning: you are updating degrees when the gradient type is not linear. This will change the gradients type which may be undesired');
        }
    };
    var setSolid = function (startingColor) {
        var newValue = startingColor || defaultColor;
        onChange(newValue);
    };
    var setGradient = function (startingGradiant) {
        var newValue = startingGradiant || defaultGradient;
        onChange(newValue);
    };
    var createGradientStr = function (newColors) {
        var sorted = newColors.sort(function (a, b) { return a.left - b.left; });
        var colorString = sorted === null || sorted === void 0 ? void 0 : sorted.map(function (cc) { return "".concat(cc === null || cc === void 0 ? void 0 : cc.value, " ").concat(cc.left, "%"); });
        onChange("".concat(gradientType, "(").concat(degreeStr, ", ").concat(colorString.join(', '), ")"));
    };
    var handleGradient = function (newColor, left) {
        if (left === void 0) { left = currentLeft; }
        var remaining = colors === null || colors === void 0 ? void 0 : colors.filter(function (c) { return !isUpperCase(c.value); });
        var newColors = __spreadArray([{ value: newColor.toUpperCase(), left: left }], remaining, true);
        createGradientStr(newColors);
    };
    var handleChange = function (newColor) {
        if (isGradient) {
            handleGradient(newColor);
        }
        else {
            onChange(newColor);
        }
    };
    var setR = function (newR) {
        var newVal = formatInputValues(newR, 0, 255);
        handleChange("rgba(".concat(newVal, ", ").concat(g, ", ").concat(b, ", ").concat(a, ")"));
    };
    var setG = function (newG) {
        var newVal = formatInputValues(newG, 0, 255);
        handleChange("rgba(".concat(r, ", ").concat(newVal, ", ").concat(b, ", ").concat(a, ")"));
    };
    var setB = function (newB) {
        var newVal = formatInputValues(newB, 0, 255);
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(newVal, ", ").concat(a, ")"));
    };
    var setA = function (newA) {
        var newVal = formatInputValues(newA, 0, 100);
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(newVal / 100, ")"));
    };
    var setHue = function (newHue) {
        var newVal = formatInputValues(newHue, 0, 360);
        var tinyNew = tc({ h: newVal, s: s, l: l });
        var _a = tinyNew.toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(a, ")"));
    };
    var setSaturation = function (newSat) {
        var newVal = formatInputValues(newSat, 0, 100);
        var tinyNew = tc({ h: h, s: newVal / 100, l: l });
        var _a = tinyNew.toRgb(), r = _a.r, g = _a.g, b = _a.b;
        handleChange("rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(a, ")"));
    };
    var setLightness = function (newLight) {
        var newVal = formatInputValues(newLight, 0, 100);
        var tinyNew = tc({ h: h, s: s, l: newVal / 100 });
        if (tinyNew === null || tinyNew === void 0 ? void 0 : tinyNew.isValid()) {
            var _a = tinyNew.toRgb(), r_1 = _a.r, g_1 = _a.g, b_1 = _a.b;
            handleChange("rgba(".concat(r_1, ", ").concat(g_1, ", ").concat(b_1, ", ").concat(a, ")"));
        }
        else {
            console.log('The new color was invalid, perhaps the lightness you passed in was a decimal? Please pass the new value between 0 - 100');
        }
    };
    var valueToHSL = function () {
        return tiny.toHslString();
    };
    var valueToHSV = function () {
        return tiny.toHsvString();
    };
    var valueToHex = function () {
        return tiny.toHexString();
    };
    var valueToCmyk = function () {
        var _a = rgb2cmyk(r, g, b), c = _a.c, m = _a.m, y = _a.y, k = _a.k;
        return "cmyk(".concat(c, ", ").concat(m, ", ").concat(y, ", ").concat(k, ")");
    };
    var setSelectedPoint = function (index) {
        if (isGradient) {
            var newGradStr = colors === null || colors === void 0 ? void 0 : colors.map(function (cc, i) { return (__assign(__assign({}, cc), { value: i === index ? high(cc) : low(cc) })); });
            createGradientStr(newGradStr);
        }
        else {
            console.log('This function is only relevant when the picker is in gradient mode');
        }
    };
    var addPoint = function (left) {
        var newColors = __spreadArray(__spreadArray([], colors.map(function (c) { return (__assign(__assign({}, c), { value: low(c) })); }), true), [{ value: currentColor, left: left }], false);
        createGradientStr(newColors);
        if (!left) {
            console.log('You did not pass a stop value (left amount) for the new color point so it defaulted to 50');
        }
    };
    var deletePoint = function (index) {
        if ((colors === null || colors === void 0 ? void 0 : colors.length) > 2) {
            var pointToDelete_1 = index || selectedPoint;
            var remaining = colors === null || colors === void 0 ? void 0 : colors.filter(function (rc, i) { return i !== pointToDelete_1; });
            createGradientStr(remaining);
            if (!index) {
                console.log('You did not pass in the index of the point you wanted to delete so the function default to the currently selected point');
            }
        }
        else {
            console.log('A gradient must have atleast two colors, disable your delete button when necessary');
        }
    };
    var setPointLeft = function (left) {
        handleGradient(currentColor, formatInputValues(left, 0, 100));
    };
    var rgbaArr = [r, g, b, a];
    var hslArr = [h, s, l];
    return { setLinear: setLinear, setRadial: setRadial, setDegrees: setDegrees, setSolid: setSolid, setGradient: setGradient, setR: setR, setG: setG, setB: setB, setA: setA, setHue: setHue, setSaturation: setSaturation, setLightness: setLightness, valueToHSL: valueToHSL, valueToHSV: valueToHSV, valueToHex: valueToHex, valueToCmyk: valueToCmyk, setSelectedPoint: setSelectedPoint, addPoint: addPoint, deletePoint: deletePoint, selectedPoint: selectedPoint, isGradient: isGradient, gradientType: gradientType, degrees: degrees, setPointLeft: setPointLeft, currentLeft: currentLeft, rgbaArr: rgbaArr, hslArr: hslArr, previousColors: previousColors };
};
//# sourceMappingURL=useColorPicker.js.map